	SUBROUTINE TRANSS(TS)
**********************************************************
*  Builts transformation matrix which transforms         *
*  spherical harmonics to cubic harmonics.               *
*  up to Lmax = 3 (f - electrons) ONLY!                  *
*  Transformation matrix TS(i,mu) is given by            *
*       _   --              _                            *
*  Y   (r)= >  TS(i,mu)*Y  (r)                           *
*   1i      --           1mu                             *
*        mu=-1,0,1                                       *
**********************************************************
        IMPLICIT REAL*8 (A-H,O-Z)
        COMPLEX*16 R1(3,-1:1),R2(5,-2:2),R3(7,-3:3),TS(7,-3:3,0:3)
     &    ,WORK(7)
	DO L=0,3
	DO M=1,7  
	DO K=-3,3
	TS(M,K,L)=(0.D0,0.D0)
	ENDDO
	ENDDO
	ENDDO
C  BUILT TRANSFORMATION MATRICES
	CALL TRANS1(R1,2)
	CALL TRANS2(R2,2)
	CALL TRANS3(R3,2)
C  S-WAVE
	TS(1,0,0)=(1.D0,0.D0)
C  P-WAVE
	DO IX=1,3
	DO MY=-1,1
	TS(IX,MY,1)=R1(IX,MY)
	ENDDO
	ENDDO
C  D-WAVE
	DO IX=1,5
	DO MY=-2,2
	TS(IX,MY,2)=R2(IX,MY)
	ENDDO
	ENDDO
C  F-WAVE
	DO IX=1,7
	DO MY=-3,3
	TS(IX,MY,3)=R3(IX,MY)
	ENDDO
	ENDDO
	END



	SUBROUTINE TRANSC(TC)
**********************************************************
*  Builts transformation matrix which transforms         *
*  cubic harmonics to spherical harmonics.               *
*  up to Lmax = 3 (f - electrons) ONLY!                  *
*  Transformation matrix TC(mu,i) is given by            *
*       _   --              _                            *
*  Y   (r)= >  TC(mu,i)*Y  (r)                           *
*   1mu     --           1i                              *
*          i=xyz                                         *
**********************************************************
        IMPLICIT REAL*8 (A-H,O-Z)
        COMPLEX*16 T1(-1:1,3),T2(-2:2,5),T3(-3:3,7),TC(-3:3,7,0:3)
	DO L=0,3
	DO M=-3,3
	DO K=1,7
	TC(M,K,L)=(0.D0,0.D0)
	ENDDO
	ENDDO
	ENDDO
	CALL TRANS1(T1,1)
	CALL TRANS2(T2,1)
	CALL TRANS3(T3,1)
C  S-WAVE
	TC(0,1,0)=(1.D0,0.D0)
C  P-WAVE
	DO MY=-1,1
	DO IX=1,3
	TC(MY,IX,1)=T1(MY,IX)
	ENDDO
	ENDDO
C  D-WAVE
	DO MY=-2,2
	DO IX=1,5
	TC(MY,IX,2)=T2(MY,IX)
	ENDDO
	ENDDO
C  F-WAVE
	DO MY=-3,3
	DO IX=1,7
	TC(MY,IX,3)=T3(MY,IX)
	ENDDO
	ENDDO
	END




	SUBROUTINE TRANS1(A1,KEY)
**********************************************************
*  Generates transformation matrix from complex          *
*  spherical harmonic to cubic harmonic for l=1.         *
*                                                        *
*  Complex spherical harmonics are (after Varshalovich): *
*       _                                                *
*  Y   (r)=+1/sqrt(2)*(x-iy)/r * a                       *
*   1-1                                                  *
*       _                                                *
*  Y   (r)= z/r                * a                       *
*   1 0                                                  *
*       _                                                *
*  Y   (r)=-1/sqrt(2)*(x+iy)/r * a                       *
*   1+1                                                  *
*                                                        *
*  Cubic harmonics are:                                  *
*       _                                                *
*  Y   (r)= x/r * a                                      *
*   1 1                                                  *
*       _                                                *
*  Y   (r)= y/r * a                                      *
*   1 2                                                  *
*       _                                                *
*  Y   (r)= z/r * a                                      *
*   1 3                                                  *
*  where a=sqrt(3/pi)/2 is a normalization constant.     *
*                                                        *
*  For a=1 it is equivalent to transition between        *
*  cartesian r=(x,y,z) and cyclic r=(r  ,r  ,r  )        *
*  coordinates of vector r.           -1   0  +1         *
*                                                        *
*  Transformation matrix T(mu,i) is given by             *
*       _   --              _                            *
*  Y   (r)= >   T(mu,i)*Y  (r)                           *
*   1mu     --           1i                              *
*          i=xyz                                         *
*                                                        *
*  Returns T if key=1, returns reciprocal(T) if key=2.   *
**********************************************************
        IMPLICIT REAL*8 (A-H,O-Z)
        COMPLEX*16 T1(-1:1,3),T1R(3,-1:1),WORK(3),A1(3,3)
c mu=-1
	T1(-1,1)=1.D0/SQRT(2.D0)
	T1(-1,2)=1.D0/SQRT(2.D0)*(0.D0,-1.D0)
	T1(-1,3)=(0.D0,0.D0)
c mu=0
	T1( 0,1)=(0.D0,0.D0)
	T1( 0,2)=(0.D0,0.D0)
	T1( 0,3)=(1.D0,0.D0)
c mu=+1
	T1(+1,1)=-1.D0/SQRT(2.D0)
	T1(+1,2)=-1.D0/SQRT(2.D0)*(0.D0,1.D0)
	T1(+1,3)=(0.D0,0.D0)
C SET RECIPROCAL MATRIX
	DO M=-1,1
	DO I=1,3
	T1R(I,M)=DCONJG(T1(M,I))
	ENDDO
	ENDDO
C TEST
C  	CALL CINV(3,T1,3,WORK,IERR)
C	DO M=-1,1
C	DO I=1,3
C	PRINT*,ABS(T1R(I,M)-T1(I-2,M+2))
C	T1R(I,M)=T1(I-2,M+2)
C	ENDDO
C	ENDDO
	IF(KEY.EQ.1)THEN
C  RETURN DIRECT MATRIX
	DO M=-1,1
	DO I=1,3
	A1(M+2,I)=T1(M,I)
	ENDDO
	ENDDO
	ELSEIF(KEY.EQ.2)THEN
C  RETURN RECIPROCAL MATRIX
	DO M=-1,1
	DO I=1,3
	A1(I,M+2)=T1R(I,M)
	ENDDO
	ENDDO
	ENDIF
        RETURN
        END



        SUBROUTINE TRANS2(A2,KEY)
**********************************************************
*  Generates transformation matrix from complex          *
*  spherical harmonic to cubic harmonic for l=2.         *
*                                                        *
*  Complex spherical harmonics are (after Varshalovich)  *
*       _                                                *
*  Y   (r)=+1/sqrt(2)*(x^2-y^2-2ixy)/r^2 * a             *
*   2-2                                                  *
*       _                                                *
*  Y   (r)=+1/sqrt(2)*(2zx-2izy)/r^2     * a             *
*   2-1                                                  *
*       _                                                *
*  Y   (r)=(3z^2/r^2-1)/sqrt(3)          * a             *
*   2 0                                                  *
*       _                                                *
*  Y   (r)=-1/sqrt(2)*(2zx+2izy)/r^2     * a             *
*   2 1                                                  *
*       _                                                *
*  Y   (r)=+1/sqrt(2)*(x^2-y^2+2ixy)/r^2 * a             *
*   2 2                                                  *
*                                                        *
*  Cubic harmonics are:                                  *
*       _                                                *
*  Y   (r)= 2yz/r^2              * a                     *
*   2 1                                                  *
*       _                                                *
*  Y   (r)= 2zx/r^2              * a                     *
*   2 2                                                  *
*       _                                                *
*  Y   (r)= 2xy/r^2              * a                     *
*   2 3                                                  *
*       _                                                *
*  Y   (r)= (x^2-y^2)/r^2        * a                     *
*   2 4                                                  *
*       _                                                *
*  Y   (r)= (3z^2/r^2-1)/sqrt(3) * a                     *
*   2 5                                                  *
*                                                        *
*  where a=sqrt(3*5/pi)/4 is a normalization constant.   *
*                                                        *
*  Transformation matrix T(mu,i) is given by             *
*       _   --              _                            *
*  Y   (r)= >   T(mu,i)*Y  (r)                           *
*   2mu     --           2i                              *
*          i=xyz                                         *
*                                                        *
*  Returns T if key=1, returns reciprocal(T) if key=2.   *
**********************************************************
        IMPLICIT REAL*8 (A-H,O-Z)
        COMPLEX*16 T2(-2:2,5),T2R(5,-2:2),WORK(5),A2(5,5)
c mu=-2
	T2(-2,1)=(0.D0,0.D0)
	T2(-2,2)=(0.D0,0.D0)
	T2(-2,3)=+1.D0/SQRT(2.D0)*(0.D0,-1.D0)
	T2(-2,4)=+1.D0/SQRT(2.D0)
	T2(-2,5)=(0.D0,0.D0)
c mu=+2
	T2(+2,1)=(0.D0,0.D0)
	T2(+2,2)=(0.D0,0.D0)
	T2(+2,3)=+1.D0/SQRT(2.D0)*(0.D0,+1.D0)
	T2(+2,4)=+1.D0/SQRT(2.D0)
	T2(+2,5)=(0.D0,0.D0)
c mu=-1
	T2(-1,1)=+1.D0/SQRT(2.D0)*(0.D0,-1.D0)
	T2(-1,2)=+1.D0/SQRT(2.D0)
	T2(-1,3)=(0.D0,0.D0)
	T2(-1,4)=(0.D0,0.D0)
	T2(-1,5)=(0.D0,0.D0)
c mu=+1
	T2(+1,1)=-1.D0/SQRT(2.D0)*(0.D0,+1.D0)
	T2(+1,2)=-1.D0/SQRT(2.D0)
	T2(+1,3)=(0.D0,0.D0)
	T2(+1,4)=(0.D0,0.D0)
	T2(+1,5)=(0.D0,0.D0)
c mu= 0
	T2( 0,1)=(0.D0,0.D0)
	T2( 0,2)=(0.D0,0.D0)
	T2( 0,3)=(0.D0,0.D0)
	T2( 0,4)=(0.D0,0.D0)
	T2( 0,5)=(1.D0,0.D0)
C SET RECIPROCAL MATRIX
	DO M=-2,2
	DO I=1,5
	T2R(I,M)=DCONJG(T2(M,I))
	ENDDO
	ENDDO
C TEST
C  	CALL CINV(5,T2,5,WORK,IERR)
C	DO M=-2,2
C	DO I=1,5
C	PRINT*,ABS(T2R(I,M)-T2(I-3,M+3))
C	T2R(I,M)=T2(I-3,M+3)
C	ENDDO
C	ENDDO
	IF(KEY.EQ.1)THEN
C  RETURN DIRECT MATRIX
	DO M=-2,2
	DO I=1,5
	A2(M+3,I)=T2(M,I)
	ENDDO
	ENDDO
	ELSEIF(KEY.EQ.2)THEN
C  RETURN RECIPROCAL MATRIX
	DO M=-2,2
	DO I=1,5
	A2(I,M+3)=T2R(I,M)
	ENDDO
	ENDDO
	ENDIF
        RETURN
        END


	SUBROUTINE TRANS3(A3,KEY)
**********************************************************
*  Generates transformation matrix from complex          *
*  spherical harmonic to cubic harmonic for l=3.         *
*                                                        *
*  Complex spherical harmonics are (after Varshalovich)  *
*       _                                                *
*  Y   (r)=+sqrt( 5/16)*(x-iy)^3/r^3                * a  *
*   3-3                                                  *
*       _                                                *
*  Y   (r)=+sqrt(15/ 8)*z(x-iy)^2/r^3               * a  *
*   3-2                                                  *
*       _                                                *
*  Y   (r)=+sqrt( 3/16)*(5z^2-r^2)(x-iy)^2/r^3      * a  *
*   3-1                                                  *
*       _                                                *
*  Y   (r)=+sqrt( 1/ 4)*(5z^2-3r^2)z/r^3            * a  *
*   3 0                                                  *
*       _                                                *
*  Y   (r)=-sqrt( 3/16)*(5z^2-r^2)(x+iy)^2/r^3      * a  *
*   3+1                                                  *
*       _                                                *
*  Y   (r)=+sqrt(15/ 8)*z(x+iy)^2/r^3               * a  *
*   3+2                                                  *
*       _                                                *
*  Y   (r)=-sqrt( 5/16)*(x+iy)^3/r^3                * a  *
*   3+3                                                  *
*                                                        *
*  Cubic harmonics are:                                  *
*       _                                                *
*  Y   (r)=+sqrt( 1/ 4)*(5x^2-3r^2)x/r^3            * a  *
*   3 1                                                  *
*       _                                                *
*  Y   (r)=+sqrt( 1/ 4)*(5y^2-3r^2)y/r^3            * a  *
*   3 2                                                  *
*       _                                                *
*  Y   (r)=+sqrt( 1/ 4)*(5z^2-3r^2)z/r^3            * a  *
*   3 3                                                  *
*       _                                                *
*  Y   (r)=+sqrt(15/ 4)*(x^2-z^2)y/r^3              * a  *
*   3 4                                                  *
*       _                                                *
*  Y   (r)=+sqrt(15/ 4)*(x^2-y^2)z/r^3              * a  *
*   3 5                                                  *
*       _                                                *
*  Y   (r)=+sqrt(15/ 4)*(y^2-z^2)x/r^3              * a  *
*   3 6                                                  *
*       _                                                *
*  Y   (r)=+sqrt(15   )*xyz/r^3                     * a  *
*   3 7                                                  *
*                                                        *
*  where a=sqrt(7/4/pi) is a normalization constant.     *
*                                                        *
*  Transformation matrix T(mu,i) is given by             *
*       _   --              _                            *
*  Y   (r)= >   T(mu,i)*Y  (r)                           *
*   2mu     --           2i                              *
*          i=xyz                                         *
*                                                        *
*  Returns T if key=1, returns reciprocal(T) if key=2.   *
**********************************************************
        IMPLICIT REAL*8 (A-H,O-Z)
        COMPLEX*16 T3(-3:3,7),T3R(7,-3:3),WORK(7),A3(7,7)
C SET RECIPROCAL TRANSFORMATION FIRST
	T3R(1,-3)=+sqrt(5.d0/16.d0)*(1.d0,0.d0)
	T3R(1,-2)=(0.d0,0.d0)
	T3R(1,-1)=-sqrt(3.d0/16.d0)*(1.d0,0.d0)
	T3R(1, 0)=(0.d0,0.d0)
	T3R(1,+1)=+sqrt(3.d0/16.d0)*(1.d0,0.d0)
	T3R(1,+2)=(0.d0,0.d0)
	T3R(1,+3)=-sqrt(5.d0/16.d0)*(1.d0,0.d0)
C
	T3R(2,-3)=-sqrt(5.d0/16.d0)*(0.d0,1.d0)
	T3R(2,-2)=(0.d0,0.d0)
	T3R(2,-1)=-sqrt(3.d0/16.d0)*(0.d0,1.d0)
	T3R(2, 0)=(0.d0,0.d0)
	T3R(2,+1)=-sqrt(3.d0/16.d0)*(0.d0,1.d0)
	T3R(2,+2)=(0.d0,0.d0)
	T3R(2,+3)=-sqrt(5.d0/16.d0)*(0.d0,1.d0)
C
	T3R(3,-3)=(0.d0,0.d0)
	T3R(3,-2)=(0.d0,0.d0)
	T3R(3,-1)=(0.d0,0.d0)
	T3R(3, 0)=(1.d0,0.d0)
	T3R(3,+1)=(0.d0,0.d0)
	T3R(3,+2)=(0.d0,0.d0)
	T3R(3,+3)=(0.d0,0.d0)
C
	T3R(4,-3)=+sqrt(3.d0/16.d0)*(0.d0,1.d0)
	T3R(4,-2)=(0.d0,0.d0)
	T3R(4,-1)=-sqrt(5.d0/16.d0)*(0.d0,1.d0)
	T3R(4, 0)=(0.d0,0.d0)
	T3R(4,+1)=-sqrt(5.d0/16.d0)*(0.d0,1.d0)
	T3R(4,+2)=(0.d0,0.d0)
	T3R(4,+3)=+sqrt(3.d0/16.d0)*(0.d0,1.d0)
C
	T3R(5,-3)=(0.d0,0.d0)
	T3R(5,-2)=+sqrt(1.d0/2.0d0)*(1.d0,0.d0)
	T3R(5,-1)=(0.d0,0.d0)
	T3R(5, 0)=(0.d0,0.d0)
	T3R(5,+1)=(0.d0,0.d0)
	T3R(5,+2)=+sqrt(1.d0/2.0d0)*(1.d0,0.d0)
	T3R(5,+3)=(0.d0,0.d0)
C
	T3R(6,-3)=-sqrt(3.d0/16.d0)*(1.d0,0.d0)
	T3R(6,-2)=(0.d0,0.d0)
	T3R(6,-1)=-sqrt(5.d0/16.d0)*(1.d0,0.d0)
	T3R(6, 0)=(0.d0,0.d0)
	T3R(6,+1)=+sqrt(5.d0/16.d0)*(1.d0,0.d0)
	T3R(6,+2)=(0.d0,0.d0)
	T3R(6,+3)=+sqrt(3.d0/16.d0)*(1.d0,0.d0)
C
	T3R(7,-3)=(0.d0,0.d0)
	T3R(7,-2)=+sqrt(1.d0/2.0d0)*(0.d0,1.d0)
	T3R(7,-1)=(0.d0,0.d0)
	T3R(7, 0)=(0.d0,0.d0)
	T3R(7,+1)=(0.d0,0.d0)
	T3R(7,+2)=-sqrt(1.d0/2.0d0)*(0.d0,1.d0)
	T3R(7,+3)=(0.d0,0.d0)
C
	DO M=-3,3
	DO I=1,7
	T3(M,I)=DCONJG(T3R(I,M))
	ENDDO
	ENDDO
C  TEST
C  	CALL CINV(7,T3R,7,WORK,IERR)
C	DO M=-3,3
C	DO I=1,7
C	PRINT*,ABS(T3R(M+4,I-4)-T3(M,I))
C	T3(M,I)=T3R(M+4,I-4)
C	ENDDO
C	ENDDO
	IF(KEY.EQ.1)THEN
C  RETURN DIRECT MATRIX
	DO M=-3,3
	DO I=1,7
	A3(M+4,I)=T3(M,I)
	ENDDO
	ENDDO
	ELSEIF(KEY.EQ.2)THEN
C  RETURN RECIPROCAL MATRIX
	DO M=-3,3
	DO I=1,7
	A3(I,M+4)=T3R(I,M)
	ENDDO
	ENDDO
	ENDIF
	END







	SUBROUTINE INTCUBC(TCUBC)
**************************************************************
*   Initializes character string.                            *
**************************************************************
	CHARACTER TCUBC(-3:3,0:3,2)*10
	DO I=1,2
	DO L=0,3
	DO M=-3,3
	TCUBC( M,L,I)='  error!  '
	ENDDO
	ENDDO
	ENDDO
C Cubic harmonics
C S-wave
	TCUBC( 0,0,1)='     s    '
C P-wave
	TCUBC(-1,1,1)='     x    '
	TCUBC( 0,1,1)='     y    '
	TCUBC(+1,1,1)='     z    '
C D-wave
	TCUBC(-2,2,1)='     yz   '
	TCUBC(-1,2,1)='     zx   '
	TCUBC( 0,2,1)='     xy   '
	TCUBC(+1,2,1)='   x2-y2  '
	TCUBC(+2,2,1)='   3z2-1  '
C F-wave
	TCUBC(-3,3,1)=' x(5x2-3) '
	TCUBC(-2,3,1)=' y(5y2-3) '
	TCUBC(-1,3,1)=' z(5z2-3) '
	TCUBC( 0,3,1)=' y(x2-z2) '
	TCUBC( 1,3,1)=' z(x2-y2) '
	TCUBC(+2,3,1)=' x(y2-z2) '
	TCUBC(+3,3,1)='    xyz   '
C Spherical harmonics:
C S-wave
	TCUBC( 0,0,2)='     s    '
C P-wave
	TCUBC(-1,1,2)='    -1    '
	TCUBC( 0,1,2)='     0    '
	TCUBC(+1,1,2)='    +1    '
C D-wave
	TCUBC(-2,2,2)='    -2    '
	TCUBC(-1,2,2)='    -1    '
	TCUBC( 0,2,2)='     0    '
	TCUBC(+1,2,2)='    +1    '
	TCUBC(+2,2,2)='    +2    '
C F-wave
	TCUBC(-3,3,2)='    -3    '
	TCUBC(-2,3,2)='    -2    '
	TCUBC(-1,3,2)='    -1    '
	TCUBC( 0,3,2)='     0    '
	TCUBC(+1,3,2)='    +1    '
	TCUBC(+2,3,2)='    +2    '
	TCUBC(+3,3,2)='    +3    '
	END
