        SUBROUTINE GGA91EXCH(RHO,GRHO,G2RHO,ALRHO,EXLDA,VXLDA,
     &                       EXGGA,VXGGA,DVX2GGA,DVXLGGA)
******************************************************************
*  Returns GGA91 correction to the exchange energy/potential.    *
*  for non-spin polarized system. For both spins call first      *
*  for rho=2*rho_up, then for rho=2*rho_dn.                      *
*  This program works with J.Perdew subroutines.                 *
****************************************************************** 
	IMPLICIT REAL*8 (A-H,O-Z)
	COMPLEX*16 GRHO(-1:1),G2RHO(-1:1,-1:1),DVX2GGA(-1:1,-1:1)
     	DATA PI/3.141592653589793D0/
        EXGGA=0.D0
        VXGGA=0.D0
C Check case rho=0
        IF(RHO.LE.1.D-09)RETURN
C Set |Grad(Rho)|
        ABSG=DCONJG(GRHO(-1))*(GRHO(-1))+
     &       DCONJG(GRHO( 0))*(GRHO( 0))+
     &       DCONJG(GRHO(+1))*(GRHO(+1))
        ABSG=SQRT(ABSG)
C Set Grad(Rho)*Grad(|Grad(Rho)|)
	ABSG2=0.D0
	DO MY=-1,1
	ABSG2=ABSG2+DCONJG(GRHO(MY))*(
     &              G2RHO(MY,-1)*DCONJG(GRHO(-1))+
     &              G2RHO(MY, 0)*DCONJG(GRHO( 0))+
     &              G2RHO(MY,+1)*DCONJG(GRHO(+1)))
	ENDDO
	ABSG2=ABSG2/ABSG
C Prepare to call Perdew program:
	D=RHO
        FK=(3.D0*PI*PI*D)**(1.D0/3.D0)
        S=ABSG/(2.D0*FK*D)
	U=ABSG2/(D*D*(2.D0*FK)**3)
        V=ALRHO/(D*(2.D0*FK)**2)
C Call Perdew program:
	CALL PERDEXCH(D,S,U,V,EX,VX,VX2,VXL)
C Return in Rydbergs (Hartrees inside Perdew's program)
	EXGGA=EX*2.D0
	VXGGA=VX*2.D0
C Set contributions from the second gradeints
	DVXLGGA=2.D0*VXL/(D*(2.D0*FK)**2) ! 2 due to Ry.
	DVX2FAC=2.D0*VX2/ABSG/(D*D*(2.D0*FK)**3)
	DO MY1=-1,1
	DO MY =-1,1
	DVX2GGA(MY1,MY)=
     &  DCONJG(GRHO(MY1))*DVX2FAC*DCONJG(GRHO(MY))
	ENDDO
	ENDDO
	END



        SUBROUTINE GGA91CORR(ROUP,RODN,GRUP,GRDN,G2UP,G2DN,
     &  ALUP,ALDN,ECLDA,VCLDAUP,VCLDADN,ECGGA,VCGGAUP,VCGGADN,
     &  DVC2GGAUP,DVC2GGADN,DVCLGGAUP,DVCLGGADN)
******************************************************************
*  Returns GGA91 correction to the correlation energy/potential. *
*  This program works with J.Perdew subroutines.                 *
******************************************************************
	IMPLICIT REAL*8 (A-H,O-Z)
        COMPLEX*16 
     &    GRUP(-1:1),G2UP(-1:1,-1:1),DVC2GGAUP(-1:1,-1:1)
     &   ,GRDN(-1:1),G2DN(-1:1,-1:1),DVC2GGADN(-1:1,-1:1)
     	DATA PI/3.141592653589793D0/
        ECGGA=0.D0
        VCGGAUP=0.D0
        VCGGADN=0.D0
C Check case rho=0
        RHO=ROUP+RODN
        IF(RHO.LE.1.D-09)RETURN
C Set |Grad(Rho)|
        ABSG=DCONJG(GRUP(-1)+GRDN(-1))*(GRUP(-1)+GRDN(-1))+
     &       DCONJG(GRUP( 0)+GRDN( 0))*(GRUP( 0)+GRDN( 0))+
     &       DCONJG(GRUP(+1)+GRDN(+1))*(GRUP(+1)+GRDN(+1))
        ABSG=SQRT(ABSG)
C Set Grad(Rho)*Grad(M)
        GRDM=DCONJG(GRUP(-1)+GRDN(-1))*(GRUP(-1)-GRDN(-1))+
     &       DCONJG(GRUP( 0)+GRDN( 0))*(GRUP( 0)-GRDN( 0))+
     &       DCONJG(GRUP(+1)+GRDN(+1))*(GRUP(+1)-GRDN(+1))
C Set Grad(Rho)*Grad(|Grad(Rho)|)
	ABSG2=0.D0
	DO MY=-1,1
	ABSG2=ABSG2+DCONJG(GRUP(MY)+GRDN(MY))*
     &  ((G2UP(MY,-1)+G2DN(MY,-1))*DCONJG(GRUP(-1)+GRDN(-1))+
     &   (G2UP(MY, 0)+G2DN(MY, 0))*DCONJG(GRUP( 0)+GRDN( 0))+
     &   (G2UP(MY,+1)+G2DN(MY,+1))*DCONJG(GRUP(+1)+GRDN(+1)))
	ENDDO
	ABSG2=ABSG2/ABSG
C Total Laplasian
        ALPS=ALUP+ALDN
C Prepare to call Perdew program:
	D=ROUP+RODN
        ZET=(ROUP-RODN)/D
        RS=(3.D0/(4.D0*PI*D))**(1.D0/3.D0)
	DDRS=-9.D0/(4.D0*PI*RS**4)
        FK=(3.D0*PI*PI*D)**(1.D0/3.D0)
        SK=SQRT(4.D0*FK/PI)
        G=((1.D0+ZET)**(2.D0/3.D0)+(1.D0-ZET)**(2.D0/3.D0))/2.D0
        T=ABSG/(2.D0*G*SK*D)
	UU=ABSG2/(D*D*(2.D0*SK*G)**3)
	VV=ALPS/(D*(2.D0*SK*G)**2)
	WW=(GRDM/D-ZET*ABSG**2/D)/(D*(2*SK*G)**2)
C EC=Correlation energy in LDA, ECZET=dEC/dZET(rho=const), 
C ECRS=dEC/dRS(zet=const.!); Hartree units now:
	EC=ECLDA*0.5D0
	ECZET=0.25D0*(VCLDAUP-ECLDA)-0.25D0*(VCLDADN-ECLDA)
	ECRS=0.25D0*((VCLDAUP-ECLDA)*(1.D0+ZET)+
     &               (VCLDADN-ECLDA)*(1.D0-ZET))/D*DDRS
C Call Perdew program:
	CALL PERDCORR(RS,ZET,T,UU,VV,WW,H,VCUP,VCDN,
     &  VC2UP,VC2DN,VCLUP,VCLDN,FK,SK,G,EC,ECRS,ECZET) 
C Return in Rydbergs (Hartrees inside Perdew's program)
	ECGGA=H*2.D0
	VCGGAUP=VCUP*2.D0
	VCGGADN=VCDN*2.D0
C Set contributions from the second gradeints
	DVCLGGAUP=2.D0*VCLUP/(D*(2.D0*SK*G)**2) ! 2 due to Ry.
	DVCLGGADN=2.D0*VCLDN/(D*(2.D0*SK*G)**2) ! 2 due to Ry.
	DVC2FACUP=2.D0*VC2UP/ABSG/(D*D*(2.D0*SK*G)**3)
	DVC2FACDN=2.D0*VC2DN/ABSG/(D*D*(2.D0*SK*G)**3)
	DO MY1=-1,1
	DO MY =-1,1
	DVC2GGAUP(MY1,MY)=DCONJG(GRUP(MY1)+GRDN(MY1))*
     &          DVC2FACUP*DCONJG(GRUP(MY )+GRDN(MY ))
	DVC2GGADN(MY1,MY)=DCONJG(GRUP(MY1)+GRDN(MY1))*
     &          DVC2FACDN*DCONJG(GRUP(MY )+GRDN(MY ))
	ENDDO
	ENDDO
	END





	SUBROUTINE PERDEXCH(D,S,U,V,EX,VX,VX2,VXL)
******************************************************************
*  GGA91 EXCHANGE FOR A SPIN-UNPOLARIZED ELECTRONIC SYSTEM       *
*  INPUT D : DENSITY                                             *
*  INPUT S:  ABS(GRAD D)/(2*KF*D)                                *
*  INPUT U:  (GRAD D)*GRAD(ABS(GRAD D))/(D**2 * (2*KF)**3)       *
*  INPUT V: (LAPLACIAN D)/(D*(2*KF)**2)                          *
*  OUTPUT:  EXCHANGE ENERGY PER ELECTRON (EX) AND POTENTIAL (VX) *
*  OUTPUT:  CONTRIBUTIONS TO VX FROM U AND V (added)             *
******************************************************************
        IMPLICIT REAL*8 (A-H,O-Z)
        DATA A1,A2,A3,A4/0.19645D0,0.27430D0,0.15084D0,100.D0/
        DATA AX,A,B1/-0.738558766382022D0,7.7956D0,0.004D0/
        DATA THRD,THRD4/0.33333333333333D0,1.33333333333333D0/
        FAC = AX*D**THRD
        S2 = S*S
        S3 = S2*S
        S4 = S2*S2
        P0 = 1.D0/SQRT(1.D0+A*A*S2)
        P1 = LOG(A*S+1.D0/P0)
        P2 = EXP(-A4*S2)
        P3 = 1.D0/(1.D0+A1*S*P1+B1*S4)
        P4 = 1.D0+A1*S*P1+(A2-A3*P2)*S2
        F = P3*P4
        EX = FAC*F-FAC ! without local contribution
C  LOCAL EXCHANGE OPTION
C       EX = FAC
C  ENERGY DONE. NOW THE POTENTIAL:
        P5 = B1*S2-(A2-A3*P2)
        P6 = A1*S*(P1+A*S*P0)
        P7 = 2.D0*(A2-A3*P2)+2.D0*A3*A4*S2*P2-4.D0*B1*S2*F
        FS = P3*(P3*P5*P6+P7)
        P8 = 2.D0*S*(B1-A3*A4*P2)
        P9 = A1*P1+A*A1*S*P0*(3.D0-A*A*S2*P0*P0)
        P10 = 4.D0*A3*A4*S*P2*(2.D0-A4*S2)-8.D0*B1*S*F-4.D0*B1*S3*FS
        P11 = -P3*P3*(A1*P1+A*A1*S*P0+4.D0*B1*S3)
        FSS = P3*P3*(P5*P9+P6*P8)+2.D0*P3*P5*P6*P11+P3*P10+P7*P11
        VX = FAC*(THRD4*F-(U-THRD4*S3)*FSS-V*FS)-FAC*THRD4 ! without local part
C  CONTRIBUTIONS TO VX FROM SECOND-ORDER GRADIENTS (ADDED)
	VX2 = -FAC*FSS
	VXL = -FAC*FS
C  LOCAL EXCHANGE OPTION:
C       VX = FAC*THRD4
        RETURN
        END




        SUBROUTINE PERDCORR(RS,ZET,T,UU,VV,WW,H,DVCUP,DVCDN,
     &  VC2UP,VC2DN,VCLUP,VCLDN,FK,SK,G,EC,ECRS,ECZET)
******************************************************************
*  GGA91 CORRELATION                                             *
*  INPUT RS: SEITZ RADIUS                                        *
*  INPUT ZET: RELATIVE SPIN POLARIZATION                         *
*  INPUT T: ABS(GRAD D)/(D*2.*KS*G)                              *
*  INPUT UU: (GRAD D)*GRAD(ABS(GRAD D))/(D**2 * (2*KS*G)**3)     *
*  INPUT VV: (LAPLACIAN D)/(D * (2*KS*G)**2)                     *
*  INPUT WW:  (GRAD D)*(GRAD ZET)/(D * (2*KS*G)**2               *
*  OUTPUT H: NONLOCAL PART OF CORRELATION ENERGY PER ELECTRON    *
*  OUTPUT DVCUP,DVCDN:  NONLOCAL PARTS OF CORRELATION POTENTIALS *
*  OUTPUT VC2UP,VC2DN:  CONTRIBUTIONS TO VC FROM UU (added)      *
*  OUTPUT VCLUP,VCLDN:  CONTRIBUTIONS TO VC FROM VV (added)      *
******************************************************************
        IMPLICIT REAL*8 (A-H,O-Z)
        DATA XNU,CC0,CX,ALF/15.75592D0,0.004235D0,-0.001667212D0,0.09D0/
        DATA C1,C2,C3,C4/0.002568D0,0.023266D0,7.389D-6,8.723D0/
        DATA C5,C6,A4/0.472D0,7.389D-2,100.D0/
        DATA THRDM,THRD2/-0.33333333333333D0,0.66666666666667D0/
        BET = XNU*CC0
        DELT = 2.D0*ALF/BET
        G3 = G**3
        G4 = G3*G
        PON = -DELT*EC/(G3*BET)
        B = DELT/(EXP(PON)-1.D0)
        B2 = B*B
        T2 = T*T
        T4 = T2*T2
        T6 = T4*T2
        RS2 = RS*RS
        RS3 = RS2*RS
        Q4 = 1.D0+B*T2
        Q5 = 1.D0+B*T2+B2*T4
        Q6 = C1+C2*RS+C3*RS2
        Q7 = 1.D0+C4*RS+C5*RS2+C6*RS3
        CC = -CX + Q6/Q7
        R0 = (SK/FK)**2
        R1 = A4*R0*G4
        COEFF = CC-CC0-3.D0*CX/7.D0
        R2 = XNU*COEFF*G3
        R3 = DEXP(-R1*T2)
        H0 = G3*(BET/DELT)*DLOG(1.D0+DELT*Q4*T2/Q5)
        H1 = R3*R2*T2
        H = H0+H1
C  LOCAL CORRELATION OPTION:
C       H = 0.0D0
C  ENERGY DONE. NOW THE POTENTIAL:
        CCRS = (C2+2.D0*C3*RS)/Q7 -
     &  Q6*(C4+2.D0*C5*RS+3.D0*C6*RS2)/Q7**2
        RSTHRD = RS/3.D0
        R4 = RSTHRD*CCRS/COEFF
        GZ = ((1.D0+ZET)**THRDM - (1.D0-ZET)**THRDM)/3.D0
        FAC = DELT/B+1.D0
        BG = -3.D0*B2*EC*FAC/(BET*G4)
        BEC = B2*FAC/(BET*G3)
        Q8 = Q5*Q5+DELT*Q4*Q5*T2
        Q9 = 1.D0+2.D0*B*T2
        Q82 = Q8*Q8
        H0B = -BET*G3*B*T6*(2.D0+B*T2)/Q8
        H0RS = -RSTHRD*H0B*BEC*ECRS
        FACT0 = 2.D0*DELT-6.D0*B
        FACT1 = Q5*Q9+Q4*Q9*Q9
        H0BT = 2.D0*BET*G3*T4*(Q4*Q5*FACT0-DELT*FACT1)/Q82
        H0RST = RSTHRD*T2*H0BT*BEC*ECRS
        H0Z = 3.D0*GZ*H0/G + H0B*(BG*GZ+BEC*ECZET)
        H0T = 2.D0*BET*G3*Q9/Q8
        H0ZT = 3.D0*GZ*H0T/G+H0BT*(BG*GZ+BEC*ECZET)
        FACT2 = Q4*Q5+B*T2*(Q4*Q9+Q5)
        FACT3 = 2.D0*B*Q5*Q9+DELT*FACT2
        H0TT = 4.D0*BET*G3*T*(2.D0*B/Q8-Q9*FACT3/Q82)
        H1RS = R3*R2*T2*(-R4+R1*T2/3.D0)
        FACT4 = 2.D0-R1*T2
        H1RST = R3*R2*T2*(2.D0*R4*(1.D0-R1*T2)-THRD2*R1*T2*FACT4)
        H1Z = GZ*R3*R2*T2*(3.D0-4.D0*R1*T2)/G
        H1T = 2.D0*R3*R2*(1.D0-R1*T2)
        H1ZT = 2.D0*GZ*R3*R2*(3.D0-11.D0*R1*T2+4.D0*R1*R1*T4)/G
        H1TT = 4.D0*R3*R2*R1*T*(-2.D0+R1*T2)
        HRS = H0RS+H1RS
        HRST = H0RST+H1RST
        HT = H0T+H1T
        HTT = H0TT+H1TT
        HZ = H0Z+H1Z
        HZT = H0ZT+H1ZT
        COMM = H+HRS+HRST+T2*HT/6.D0+7.D0*T2*T*HTT/6.D0
        PREF = HZ-GZ*T2*HT/G
        FACT5 = GZ*(2.D0*HT+T*HTT)/G
        COMM = COMM-PREF*ZET-UU*HTT-VV*HT-WW*(HZT-FACT5)
        DVCUP = COMM + PREF
        DVCDN = COMM - PREF
C  CONTRIBUTIONS TO VC FROM SECOND-ORDER GRADIENTS (ADDED)
	VC2UP = -HTT
	VC2DN = -HTT
	VCLUP = -HT
	VCLDN = -HT
C  LOCAL CORRELATION OPTION:
C       DVCUP = 0.0D0
C       DVCDN = 0.0D0
        RETURN
        END
